package se.tada.preproc;

import java.io.IOException;

class IncludeDirective extends Unit
{
	public IncludeDirective(PreProcessor owner, String fileName, int startLine)
	{
		super(owner, fileName, startLine);
		m_type = NormalInclude;
		m_block = null;
	}

	public void expand()
	throws ParseException
	{
		if(m_type == InvalidInclude)
			return;

		if(m_block == null)
		{
			String[] fp = new String[1];
			m_block = m_owner.resolveInclude(this, fp);
			m_contents = fp[0];
			if(m_block == null)
			{
				// Owner is dependency checking and the file was not
				// found.
				//
				m_type = InvalidInclude;
				return;
			}
		}

		if(m_type == NormalInclude)
			m_owner.addDependency(m_contents);

		int top = m_block.length;
		if(top > 0)
		{
			m_owner.pushFile(m_contents, 1);
			for(int idx = 0; idx < top; ++idx)
				m_block[idx].expand();
			m_owner.popFile();
		}
		try
		{
			m_owner.write("\n");
		}
		catch(IOException e)
		{
			throw new ParseException(e);
		}
	}

	public boolean isSignificant()
	{
		return m_type != InvalidInclude;
	}

	public int parse(char[] buf, int bIdx)
	{
		// Scan until next nonescaped newline.
		//
		char c;
		char terminator;
		int start;
		int[] bPos = new int[]{bIdx};
		String line = this.cleanDirectiveLine(buf, bPos);
		bIdx = bPos[0];

		char[] rBuf = line.toCharArray();
		int rIdx = 0;
		int rTop = rBuf.length;

		c = (rTop == 0) ? 0 : rBuf[rIdx++];
		if(c == '"')
		{
			m_type = NormalInclude;
			terminator = c;
		}
		else if(c == '<')
		{
			m_type = SystemInclude;
			terminator = '>';
		}
		else
		{
			this.parseError("bad include syntax");
			m_type = InvalidInclude;
			bIdx = skipToStartOfLine(buf, bIdx, bPos);
			m_lineCount += bPos[0];
			return bIdx;
		}

		start = rIdx;
		for(;;)
		{
			if(rIdx >= rTop)
			{
				this.parseError("bad include syntax");
				m_type = InvalidInclude;
				return bIdx;
			}

			switch(c = rBuf[rIdx++])
			{
				case ESCNL_MARKER:
				case '\n':
					this.parseError("bad include syntax");
					m_type = InvalidInclude;
					m_lineCount++;
					break;

				case '"':
				case '>':
					if(terminator == c)
					{
						--rIdx;
						m_contents = String.valueOf(rBuf, start, rIdx - start);
					}
					else
						this.parseError("bad include syntax");
					break;

				default:
					continue;
			}
			break;
		}
		return bIdx;
	}

	static public final int	InvalidInclude	= -1;
	static public final int	NormalInclude	= 0;
	static public final int	SystemInclude	= 1;

	private int				m_type;

	private Unit[]			m_block;
}