package se.tada.preproc;

abstract class Unit implements MarkerTypes
{
	protected Unit(PreProcessor owner, String fileName, int startLine)
	{
		m_owner = owner;
		m_fileName = fileName;
		m_startLine = startLine;
		m_contents = "";
		m_lineCount = 0;
	}

	public String cleanDirectiveLine(char[] buf, int[] bufPos)
	{
		// Scan until next nonescaped newline.
		//
		char c;
		int[] lc = new int[]{0};
		int bPos = skipWhite(buf, bufPos[0], lc);
		int bTop = buf.length;
		int start = bPos;
		int rPos = bPos;

		m_lineCount += lc[0];

		// Look out for comments.
		//
		while(bPos < bTop)
		{
			c = buf[bPos++];
			switch(c)
			{
				case '/':
					if(bPos == bTop)
					{
						buf[rPos++] = '/';
						break;
					}

					c = buf[bPos++];
					if(c == '/')
					{
						bPos = skipToStartOfLine(buf, bPos, lc);
						m_lineCount += lc[0];
						break;
					}

					if(c == '*')
					{
						boolean[] ok = new boolean[]{true};
						bPos = skipCComment(buf, bPos + 2, lc, ok);
						if(!ok[0])
							this.parseError("unterminated comment");

						// A comment within a directive always generates
						// a whitespace.
						//
						m_lineCount += lc[0];
						buf[rPos++] = ' ';
						continue;
					}
					buf[rPos++] = '/';
					buf[rPos++] = c;
					continue;

				case ESCNL_MARKER:
					++m_lineCount;
					buf[rPos++] = ' ';
					continue;

				default:
					buf[rPos++] = c;
					continue;

				case '\n':
					++m_lineCount;
			}
			break;
		}

		--rPos;
		while((c = buf[rPos]) == ' ' || c == '\t' || c == '\f')
			--rPos;
		++rPos;
		bufPos[0] = bPos;
		return String.valueOf(buf, start, rPos - start);
	}

	public void expand()
	throws ParseException
	{
		// Default expansion. Just putout a #line directive
		//
		m_owner.setLine(m_startLine + m_lineCount);
	}

	public String getContents()
	{
		return m_contents;
	}

	public String getFileName()
	{
		return m_fileName;
	}

	public int getStartLine()
	{
		return m_startLine;
	}

	static public String getIdentifier(char[] buf, int[] bPos)
	{
		// Copy the name of the directive from the input.
		//
		int bTop = buf.length;
		int bIdx = bPos[0];
		int start = bIdx++;

		while(bIdx < bTop)
		{
			char c = buf[bIdx];
			if(!(Character.isLetterOrDigit(c) || c == '_'))
				break;
			++bIdx;
		}
		String v = String.valueOf(buf, start, bIdx - start);
		bPos[0] = bIdx;
		return v;
	}

	public int getLineCount()
	{
		return m_lineCount;
	}

	public boolean isSignificant()
	{
		return true;
	}

	public void parseError(String aString)
	{
		m_owner.parseError(m_fileName, m_startLine + m_lineCount, aString);
	}

	public void setDirectiveName(String v)
	{}

	static public int skipCComment(char[] buf, int bIdx, int[] lineCnt,
			boolean[] ok)
	{
		char c;
		int bTop = buf.length;

		lineCnt[0] = 0;
		ok[0] = true;
		for(;;)
		{
			if(bIdx >= bTop)
			{
				--bIdx;
				ok[0] = false;
				break;
			}

			switch(c = buf[bIdx++])
			{
				case '\\':
					if(bIdx < bTop)
					{
						c = buf[bIdx++];
						if(c == '\n')
							lineCnt[0]++;
					}
					continue;

				case '*':
					if(bIdx < bTop)
					{
						c = buf[bIdx];
						if(c == '/')
						{
							++bIdx;
							break;
						}
					}
					continue;

				case '\n':
					lineCnt[0]++;

				// Fall through to default:
				//
				default:
					continue;
			}
			break;
		}
		return bIdx;
	}

	static public int skipSingleQuote(char[] buf, int bIdx)
	{
		int bTop = buf.length;
		if(bIdx >= bTop)
			return bTop;

		char c = buf[bIdx++];
		if(c != '\\')
		{
			if(buf[bIdx] != '\'')
				++bIdx;
			return bIdx;
		}

		if(!(bIdx + 1 < bTop && buf[bIdx] == '0'))
			return bIdx + 2;

		++bIdx;
		switch(buf[bIdx++])
		{
			case 'x':
			case 'X':
				for(int i = 0; i < 2 && bIdx < bTop; ++i)
				{
					if(Character.digit(buf[bIdx], 16) == -1)
						break;
					++bIdx;
				}
				break;
			default:
				--bIdx;
				for(int i = 0; i < 3 && bIdx < bTop; ++i)
				{
					if(Character.digit(buf[bIdx], 8) == -1)
						break;
					++bIdx;
				}
		}
		return bIdx;
	}

	static public int skipToStartOfLine(char[] buf, int bIdx, int[] lineCnt)
	{
		char c = 0;
		int cnt = 0;
		int bTop = buf.length;
		while(bIdx < bTop)
		{
			c = buf[bIdx++];
			switch(c)
			{
				case ESCNL_MARKER:
					++cnt;
				// Fall through to default
				default:
					continue;

				case '\n':
					++cnt;
			}
			break;
		}

		// Reached end of file?
		// Treat EOF as implicit newline.
		//
		if(bIdx == bTop && c != '\n')
			++cnt;
		lineCnt[0] = cnt;
		return bIdx;
	}

	static public int skipWhite(char[] buf, int bIdx, int[] lineCnt)
	{
		char c;
		int bTop = buf.length;
		lineCnt[0] = 0;
		while(bIdx < bTop)
		{
			c = buf[bIdx];
			switch(c)
			{
				case ESCNL_MARKER:
					lineCnt[0]++;
					bIdx++;
					continue;

				case NULL_MARKER:
				case ' ':
				case '\t':
				case '\f':
					bIdx++;
					continue;
			}
			break;
		}
		return bIdx;
	}

	static public int skipAllWhite(char[] buf, int bIdx, int[] lineCnt)
	{
		char c;
		int bTop = buf.length;
		lineCnt[0] = 0;
		while(bIdx < bTop)
		{
			c = buf[bIdx];
			switch(c)
			{
				case NULL_MARKER:
				case ' ':
				case '\t':
				case '\f':
					bIdx++;
					continue;

				case ESCNL_MARKER:
				case '\n':
					lineCnt[0]++;
					bIdx++;
					continue;
			}
			break;
		}
		return bIdx;
	}

	static public Unit next(PreProcessor owner, char[] buf, int[] bPos,
			String currentFileName, int[] lineCount)
	throws ParseException
	{
		Unit unit = null;
		int bTop = buf.length;
		int bIdx = bPos[0];
		int[] bp = new int[1];

		for(;;)
		{
			String directiveName = null;
			if(bIdx >= bTop)
			{
				bPos[0] = bIdx;
				return null;
			}

			int line = lineCount[0];
			if(buf[bIdx] == '#')
			{
				bIdx = skipWhite(buf, ++bIdx, bp);
				line += bp[0];
				lineCount[0] = line;
				bp[0] = bIdx;
				directiveName = getIdentifier(buf, bp);
				bIdx = bp[0];
				
				if(directiveName.equals("define"))
					unit = new DefineDirective(owner, currentFileName, line);
				else if(directiveName.equals("elif"))
					unit = new ElifDirective(owner, currentFileName, line);
				else if(directiveName.equals("else"))
					unit = new ElseDirective(owner, currentFileName, line);
				else if(directiveName.equals("endif"))
					unit = new EndifDirective(owner, currentFileName, line);
				else if(directiveName.equals("error"))
					unit = new ErrorDirective(owner, currentFileName, line);
				else if(directiveName.equals("if"))
					unit = new IfDirective(owner, currentFileName, line);
				else if(directiveName.equals("ifdef"))
					unit = new IfdefDirective(owner, currentFileName, line);
				else if(directiveName.equals("ifndef"))
					unit = new IfndefDirective(owner, currentFileName, line);
				else if(directiveName.equals("include"))
					unit = new IncludeDirective(owner, currentFileName, line);
				else if(directiveName.equals("pragma"))
					unit = new PragmaDirective(owner, currentFileName, line);
				else if(directiveName.equals("undef"))
					unit = new UndefDirective(owner, currentFileName, line);
				else
					unit = new UnknownDirective(owner, currentFileName, line);

				unit.setDirectiveName(directiveName);
			}	
			else
				unit = new PlainText(owner, currentFileName, line);

			bIdx = unit.parse(buf, bIdx);
			lineCount[0] += unit.getLineCount();
			if(unit.isSignificant())
				break;
		}
		bPos[0] = bIdx;
		return unit;
	}

	public abstract int parse(char[] buf, int bIdx)
	throws ParseException;

	public void setContents(String contents)
	{
		m_contents = contents;
	}

	protected PreProcessor	m_owner;

	protected String		m_fileName;

	protected String		m_contents;

	protected int			m_startLine;

	protected int			m_lineCount;
}
