package se.tada.util;

import java.util.ArrayList;

public class StringUtils
{
	public static String expandEscape(String str)
	{
		char ch;
		final int t = str.length();
		StringBuilder out = new StringBuilder(t + 30);
		for(int i = 0; i < t; i++)
		{
			ch = str.charAt(i);
			switch(ch)
			{
				case '\b':
					out.append("\\b");
					continue;
				case '\t':
					out.append("\\t");
					continue;
				case '\n':
					out.append("\\n");
					continue;
				case '\f':
					out.append("\\f");
					continue;
				case '\r':
					out.append("\\r");
					continue;
				case '\"':
					out.append("\\\"");
					continue;
				case '\'':
					out.append("\\\'");
					continue;
				case '\\':
					out.append("\\\\");
					continue;
				default:
					if(ch < 0x20 || (ch > 0x7e && ch <= 0xff))
					{
						// Use normal notation with 3 octal digits
						//
						out.append("\\");
						String s = "000" + Integer.toString(ch, 8);
						int l = s.length();
						out.append(s.substring(l - 3, l));
						continue;
					}

					if(ch > 0xff)
					{
						// Use unicode notation with 4 hexadecimal digits
						//
						String s = "0000" + Integer.toString(ch, 16);
						int l = s.length();
						out.append("\\u");
						out.append(s.substring(l - 4, l));
						continue;
					}
			}
			out.append(ch);
		}
		return out.toString();
	}

	public static String makeGetter(String mbrName)
	{
		return makeXetter(mbrName, 'g');
	}

	public static String makeSetter(String mbrName)
	{
		return makeXetter(mbrName, 's');
	}

	private static String makeXetter(String mbrName, char gs)
	{
		final char[] nameChars = mbrName.toCharArray();
		final int len = nameChars.length;
		char[] allChars = new char[len+3];
		allChars[0] = gs;
		allChars[1] = 'e';
		allChars[2] = 't';
		allChars[3] = Character.toUpperCase(nameChars[0]);
		for(int idx = 1; idx < len; ++idx)
			allChars[idx+3] = nameChars[idx];
		return new String(allChars);
	}

	public static String parseEscape(String str)
	{
		int pos = str.indexOf('\\');
		if(pos < 0)
			return str;

		char[] cp = str.toCharArray();
		int top = cp.length;
		int idx;
		int v1, v2, v3, v4;

		for(idx = pos; pos < top; ++pos, ++idx)
		{
			char c = cp[pos];
			if(c != '\\' || pos + 1 == top)
			{
				cp[idx] = c;
				continue;
			}

			switch(c = cp[++pos])
			// skip escape and check next token
			{
				case '\\':
					cp[idx] = '\\';
					break;
				case 't':
					cp[idx] = '\t';
					break;
				case 'n':
					cp[idx] = '\n';
					break;
				case 'b':
					cp[idx] = '\b';
					break;
				case 'r':
					cp[idx] = '\r';
					break;
				case 'f':
					cp[idx] = '\f';
					break;
				case '"':
					cp[idx] = '"';
					break;
				case 'x':
				case 'X':
					if(pos + 2 < top
							&& (v1 = Character.digit(cp[pos + 1], 16)) >= 0
							&& (v2 = Character.digit(cp[pos + 2], 16)) >= 0)
					{
						pos += 2;
						cp[idx] = (char)((v1 << 4) | v2);
					}
					else
					{
						cp[idx++] = '\\';
						cp[idx] = c;
					}
					break;

				case 'u':
					if(pos + 4 < top
							&& (v1 = Character.digit(cp[pos + 1], 16)) >= 0
							&& (v2 = Character.digit(cp[pos + 2], 16)) >= 0
							&& (v3 = Character.digit(cp[pos + 2], 16)) >= 0
							&& (v4 = Character.digit(cp[pos + 2], 16)) >= 0)
					{
						pos += 4;
						cp[idx] = (char)((v1 << 12) | (v2 << 8) | (v3 << 4) | v4);
					}
					else
					{
						cp[idx++] = '\\';
						cp[idx] = c;
					}
					break;

				default:
					//
					// Perhaps an octal number? May be in the following forms
					// [0-3][0-7][0-7]
					// [0-7][0-7]
					// [0-7]
					//
					if(++pos < top && (v1 = Character.digit(cp[pos], 8)) >= 0)
					{
						if(++pos < top
								&& (v2 = Character.digit(cp[pos], 8)) >= 0)
						{
							if(++pos < top && v1 < 4
									&& (v3 = Character.digit(cp[pos], 8)) >= 0)
							{
								++pos;
								cp[idx] = (char)((v1 << 6) | (v2 << 3) | v3);
							}
							else
								cp[idx] = (char)((v1 << 3) | v2);
						}
						else
							cp[idx] = (char)v1;
					}
					else
					{
						cp[idx++] = '\\';
						cp[idx] = c;
					}
					--pos;
					break;
			}
		}
		return String.valueOf(cp, 0, idx);
	}

	public static String right(String str, int startPos)
	{
		int top = str.length();
		return (top > startPos) ? str.substring(startPos, top) : "";
	}

	public static String[] toArray(String str, String sep)
	{
		if(str == null)
			return new String[0];

		final int sepLen = sep.length();
		int idx = str.indexOf(sep);
		int start = 0;
		ArrayList v = new ArrayList();
		while(idx >= 0)
		{
			v.add(str.substring(start, idx));
			start = idx + sepLen;
			idx = str.indexOf(sep, start);
		}
		v.add(str.substring(start, str.length()));
		return (String[])v.toArray(new String[v.size()]);
	}
}