package se.tada.util.eval;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Map;

import se.tada.util.StringUtils;

public class MemberExpression extends BinaryExpression
{
	public MemberExpression(Expression e1, Expression e2)
	{
		super(e1, e2);
	}

	public Object eval(Map ctx)
	throws EvalException
	{
		Object member = this.getRHS().eval(ctx);
		if(member == null)
			return null;

		Object base = this.getLHS().eval(ctx);
		if(base == null)
			return null;

		String mbr = member.toString();
		boolean isClass = (base instanceof Class);
		Class cls = isClass ? (Class)base : base.getClass();

		try
		{
			try
			{
				Field fld = cls.getField(mbr);
				if(isClass)
				{
					if(!Modifier.isStatic(fld.getModifiers()))
						throw new NoSuchFieldException("static " + mbr);
					return fld.get(null);
				}
				return fld.get(base);
			}
			catch(NoSuchFieldException e)
			{
				Method gtr = cls.getMethod(StringUtils.makeGetter(mbr), null);
				if(isClass)
				{
					if(!Modifier.isStatic(gtr.getModifiers()))
						throw new NoSuchMethodException("static " + mbr);
					return gtr.invoke(null, null);
				}
				return gtr.invoke(base, null);
			}
		}
		catch(NoSuchMethodException e)
		{
			throw new EvalException("Unable to resolve member " + mbr + ": " + e.getMessage());
		}
		catch(InvocationTargetException e)
		{
			Throwable t = e.getTargetException();
			if(t instanceof RuntimeException)
				throw (RuntimeException)t;
			throw new EvalException(t);
		}
		catch(IllegalAccessException e)
		{
			throw new EvalException("Illegal access on member " + mbr + ": " + e.getMessage());
		}
	}
}
