package se.tada.util.printf;

import java.io.IOException;
import java.io.Writer;
import java.text.ParsePosition;

public abstract class BaseFormat extends Part
{

	private boolean m_leftAligned;

	private Object m_precision;

	private Object m_width;

	private boolean m_zeroPad;

	public void format(Object val, Writer out) throws IOException
	{
		int precision = -1;
		int width = 0;

		if(m_width == Boolean.TRUE || m_precision == Boolean.TRUE)
		{
			// The format contained '*' which means that either
			// the width or the precision should be picked from
			// the argument list. Thus, we need to have multiple
			// arguments.
			//
			Object[] args = (Object[])val;
			int argIdx = 0;

			if(m_width == Boolean.TRUE)
				width = ((Number)args[argIdx++]).intValue();
			else if(m_width instanceof Number)
				width = ((Number)m_width).intValue();

			if(m_precision == Boolean.TRUE)
				precision = ((Number)args[argIdx++]).intValue();
			else if(m_precision instanceof Number)
				precision = ((Number)m_precision).intValue();

			val = args[argIdx];
		}
		else
		{
			if(m_width instanceof Number)
				width = ((Number)m_width).intValue();

			if(m_precision instanceof Number)
				precision = ((Number)m_precision).intValue();
		}

		String v = this.basicFormat(val, width, precision);
		if(width > 0)
		{
			int len = v.length();
			if(width > len)
			{
				char[] pad = new char[1];
				pad[0] = (isZeroPad() ? '0' : ' ');
				if(m_leftAligned)
				{
					out.write(v);
					while(len < width)
					{
						out.write(pad);
						++len;
					}
				}
				else
				{
					while(len < width)
					{
						out.write(pad);
						++len;
					}
					out.write(v);
				}
				return;
			}
		}
		out.write(v);
	}

	protected abstract String basicFormat(Object val, int width, int precision);

	public int getRequiredNumberOfArguments()
	{
		int argCnt = 1;
		if(m_width == Boolean.TRUE)
			argCnt++;
		if(m_precision == Boolean.TRUE)
			argCnt++;
		return argCnt;
	}

	public boolean isZeroPad()
	{
		// Zeropad is only meaningful for right aligned quantities.
		//
		return m_zeroPad && !m_leftAligned;
	}

	public Object parseObject(String source, ParsePosition status)
	{
		return null;
	}

	public void setLeftAligned(boolean leftAligned)
	{
		m_leftAligned = leftAligned;
	}

	public void setPrecision(Object prec)
	{
		m_precision = prec;
	}

	public void setWidth(Object width)
	{
		m_width = width;
	}

	public void setZeroPad(boolean zeroPad)
	{
		m_zeroPad = zeroPad;
	}
}