package se.tada.util.printf;

public class DecimalIntFormat extends NumberFormat
{
	private static final long serialVersionUID = -6231890923071564363L;
	private final boolean m_unsigned;

	public DecimalIntFormat(int prefix, char size, boolean unsigned)
	{
		super(prefix, size);
		m_unsigned = unsigned;
	}

	protected String basicFormat(Object src, int width, int precision)
	{
		long val = m_unsigned ? this.getUnsigned(src) : this.getSigned(src);

		String v;
		if(m_unsigned && val < 0)
			v = toStringUnsigned(val, 10);
		else
			v = Long.toString(val);

		int nChars = v.length();
		boolean negate = (val < 0);

		if((negate || this.getPrefixType() == SIGN) // Sign will be printed
				&& width - 1 > precision // Padding might be done
				&& this.isZeroPad()) // We use zeroes for padding.
		{
			// Padding with zeroes should look funny if the
			// sign ended up between the padding and the
			// actual number. By increasing the precision (the
			// minimun number of digits in this case) we will
			// force the number to become adjacent to the sign.
			//
			precision = width - 1;
		}

		if(precision >= 0)
		{
			if(precision == 0)
			{
				// Special case. Only zero can become an
				// empty string.
				//
				if(val == 0)
				{
					v = "";
					nChars = 0;
				}
			}
			else
			{
				// precision for an Integer field controls the
				// minimum number of digits.
				//
				if(negate)
					--nChars;

				if(precision > nChars)
				{
					if(negate)
						// Temporarilly remove sign added by the
						// 'toString' method
						//
						v = v.substring(1, nChars + 1);

					while(precision > nChars)
					{
						// Zero pad needed. Since this is fairly
						// uncommon, and most often a question of
						// adding one or two digits, we use this
						// simple but in many cases slow method.
						//
						v = "0" + v;
						++nChars;
					}

					if(negate)
						//
						// Restore the sign.
						//
						v = "-" + v;
				}
			}
		}

		if(!negate)
			v = this.appendPrefix(v);
		return v;
	}
}