package se.tada.util.printf;

public abstract class ExtIntFormat extends NumberFormat
{
	public ExtIntFormat(int prefixType, char size)
	{
		super(prefixType, size);
	}

	protected String basicFormat(Object src, int width, int precision)
	{
		long val = this.getUnsigned(src);
		String v = this.makeString(val);
		int nChars = v.length();
		int prefixType = this.getPrefixType();
		int pfxLen = ((prefixType == OCTAL) ? 1 : ((prefixType == HEX) ? 2 : 0));

		if(pfxLen > 0 && val != 0)
		{
			if(width > precision + pfxLen && this.isZeroPad())
			{
				// Padding with zeroes should look funny if the
				// prefix ended up between the padding and the
				// actual number. By increasing the precision
				// (the minimun number of digits in this case) we
				// will force the number to become adjacent to
				// the sign.
				//
				precision = width - pfxLen;
			}
		}

		if(precision >= 0)
		{
			if(precision == 0)
			{
				// Special case. Only zero can become an
				// empty string.
				//
				if(val == 0)
				{
					v = "";
					nChars = 0;
				}
			}
			else
			{
				// precision for an Integer field controls the
				// minimum number of digits.
				//
				while(precision > nChars)
				{
					// Zero pad needed. Since this is fairly
					// uncommon, and most often a question of
					// adding one or two digits, we use this
					// simple but in many cases slow method.
					//
					v = "0" + v;
					++nChars;
				}
			}
		}
		return this.appendPrefix(v);
	}

	protected abstract String makeString(long val);
}