package se.tada.util.printf;

public abstract class NumberFormat extends BaseFormat implements PrefixType
{
	// All possible chars for representing a number as a String.
	// The digits exists in Integer but is, for some reason, not
	// available outside the lang package.
	//
	public final static char[] digits = { '0', '1', '2', '3', '4', '5', '6',
			'7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
			'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w',
			'x', 'y', 'z' };

	private final char m_size;

	private final int m_prefixType;

	protected NumberFormat(int prefixType, char size)
	{
		m_prefixType = prefixType;
		m_size = size;
	}

	protected String appendPrefix(String v)
	{
		switch(m_prefixType)
		{
			case NONE:
				break;
			case SIGN:
				v = "+" + v;
				break;
			case SPACE:
				v = " " + v;
				break;
			case OCTAL:
				v = "0" + v;
				break;
			case HEX:
				v = (this.isUpperCase() ? "0X" : "0x") + v;
		}
		return v;
	}

	protected int getPrefixType()
	{
		return m_prefixType;
	}

	protected long getSigned(Object v)
	{
		long val = ((Number)v).longValue();
		switch(m_size)
		{
			case 'b':
				val = ((byte)(val & 0xffL));
				break;
			case 'h':
				val = ((short)(val & 0xffffL));
				break;
			case 'i':
				val = ((int)(val & 0xffffffffL));
		}
		return val;
	}

	protected long getUnsigned(Object v)
	{
		long r;
		if(v instanceof Integer || v instanceof Float)
		{
			int val = ((Number)v).intValue();
			switch(m_size)
			{
				case 'b':
					r = (val & 0xff);
					break;
				case 'h':
					r = (val & 0xffff);
					break;
				case 'l':
					r = val;
					break;
				default:
					if(val >= 0)
						r = val;
					else
					{
						r = 0x80000000L;
						r |= (val & Integer.MAX_VALUE);
					}
			}
		}
		else if(v instanceof Long || v instanceof Double)
		{
			long val = ((Number)v).longValue();
			switch(m_size)
			{
				case 'b':
					r = (val & 0xffL);
					break;
				case 'h':
					r = (val & 0xffffL);
					break;
				case 'i':
					r = (val & 0xffffffffL);
					break;
				default:
					r = val;
			}
		}
		else if(v instanceof Short)
		{
			short val = ((Number)v).shortValue();
			switch(m_size)
			{
				case 'b':
					r = (val & 0xff);
					break;
				case 'i':
				case 'l':
					r = val;
					break;
				default:
					if(val >= 0)
						r = val;
					else
					{
						r = 0x8000L;
						r |= (val & Short.MAX_VALUE);
					}
			}
		}
		else if(v instanceof Byte)
		{
			byte val = ((Number)v).byteValue();
			switch(m_size)
			{
				case 'h':
				case 'i':
				case 'l':
					r = val;
					break;
				default:
					if(val >= 0)
						r = val;
					else
					{
						r = 0x80L;
						r |= (val & Byte.MAX_VALUE);
					}
			}
		}
		else
			r = this.getUnsigned(Integer.valueOf(v.toString()));
		return r;
	}

	public boolean isUpperCase()
	{
		return false;
	}

	public static String toStringUnsigned(long val, int radix)
	{
		if(val == 0)
			return "0";

		char[] buf = new char[64];
		int pos = 64;
		int hr = (radix >> 1);

		// Develop the digits of the value
		//
		char[] tab = digits;
		long max = Long.MAX_VALUE;
		while(val != 0)
		{
			int p = (int)(val & 1);
			val = (val >> 1) & max;
			buf[--pos] = tab[(int)(val % hr * 2 + p)];
			val /= hr;
		}
		return new String(buf, pos, 64 - pos);
	}
}